%******************************************************
% m_id3.m  (1 measured frequency and its eigenvector) *
% Example for identifiability in Chapter 2            *
% By KV Yuen                                          *
%******************************************************
close all
clear
% Parameters definition
A0=[2 1;
    1 1];
sigman=0.05;
[V,L]=eig(A0);
[L,i]=sort(diag(L));
L0=L(1); % actual eigenvalue (smaller)
V0=V(:,i(1)); % corresponding actual eigenvector
randn('state',sum(999*clock));
Lm=L0+sigman*L0*randn(1,5); % measurements
phim=V0(2)/V0(1)+sigman*V0(2)/V0(1)*randn(1,5); % measurements
x=[0.92:0.005:1.065]; Nx=length(x);
y=[0.91:0.005:1.18]; Ny=length(y);
% Draw posterior PDF
for pp=1:Nx
    for qq=1:Ny
        A=[x(pp)+y(qq) y(qq);
                 y(qq) y(qq)];
        [V,LAtemp]=eig(A);
        [LA,i]=sort(diag(LAtemp));
        phi=V(2,i(1))/V(1,i(1));
        J(qq,pp)=sum((LA(1)-Lm).^2)/(L0*sigman)^2+sum((phi-phim).^2)/(V0(2)/V0(1)*sigman)^2;
    end
end
p=(2*pi)^(-10/2)*(L0*sigman)^(-5)*(abs(V0(2)/V0(1))*sigman)^(-5)*exp(-J/2);
mesh(x,y,p)
xlabel('\theta_1')
ylabel('\theta_2')
zlabel('p(D|\theta,C)')

figure
contour(x,y,p)
shg
xlabel('\theta_1')
ylabel('\theta_2')