%***************************************************
% duffing.m: Identification for Duffing oscillator *
%***************************************************
clear
close all
clc
global dis T dt dtf nlnpmin Sy wpos ESa ESb K1r K3r cr M0
%**** Parameters input ****
T=400;                    % Duration of measurement
dt=0.05;                  %  Sampling time interval
M0=1;                     % Actual mass
K1r=5;K3r=1;              % Actual stiffnesses
cr=0.1;                   % Actual Damping coefficient
Sf0r=[0.02 0.04];         % Actual spectral Intensities for the white noise
nl=0.15;                  % Measurement Noise Level
%**************************

% Part 1  Data generation
disp('Data generation in progress ...')
[Sy1,sdn1]=spd_duffing(Sf0r(1),nl);
[Sy2,sdn2]=spd_duffing(Sf0r(2),nl);

% Part 2  Frequency Domain Approach
nlnpmin=9e99;dis=1;
semilogy([0:length(Sy2)-1]/T,Sy2)
xlabel('Frequency (Hz)')
ylabel('S_y_,_N')
disp('Please click at the peak of the spectrum')
[W,S]=ginput(1);
close; pause(1)
wpos=max(fix(1.2*W*T)+1,20); % ending index of the frequency index set
Sy=[Sy1(1:wpos);Sy2(1:wpos)];
clear Sy1 Sy2
dis=1;
disp('Optimization in progress...')
para0=[K1r K3r cr Sf0r sdn1^2 sdn2^2];
Npar=7; % # of parameters for identification
[para,nlnpmin]=fminsearch('pduffing',para0,optimset('MaxIter',10000,'MaxFunEvals',10000));

disp('Computing Hessian ...')
dx=para0/10;dis=0;
for p=1:Npar
    Hess(p,p)=(pduffing(para+[zeros(1,p-1) dx(p) zeros(1,Npar-p)])-2*nlnpmin+...
        pduffing(para-[zeros(1,p-1) dx(p) zeros(1,Npar-p)]))/dx(p)^2;
    for q=p+1:Npar
        Hess(p,q)=(pduffing(para+[zeros(1,p-1) dx(p) zeros(1,q-p-1) dx(q) zeros(1,Npar-q)])-...
            pduffing(para+[zeros(1,p-1) -dx(p) zeros(1,q-p-1) dx(q) zeros(1,Npar-q)])-...
            pduffing(para+[zeros(1,p-1) dx(p) zeros(1,q-p-1) -dx(q) zeros(1,Npar-q)])+...
            pduffing(para-[zeros(1,p-1) dx(p) zeros(1,q-p-1) dx(q) zeros(1,Npar-q)]))/4/dx(p)/dx(q);
        Hess(q,p)=Hess(p,q);  
    end
end
covm=inv(Hess);
SD=((diag(covm)).^0.5)';
results=[para0' para' SD' abs(SD./para)' abs((para0-para)./SD)'];
disp(['  --------------------------------------------------'])
disp(['    Actual  Identified   s.d.     c.o.v.     n.d.'])
disp(['  --------------------------------------------------'])
disp([results])

% Drawing contours
[P,S]=eig(covm(1:2,1:2));
Npl=40;
for p=1:Npl+1
    y(1,1)=(S(1,1))^0.5*cos(2*pi/Npl*p);
    y(2,1)=(S(2,2))^0.5*sin(2*pi/Npl*p);
    x(:,p)=P*y;
end
figure
plot(para(1),para(2),'o')
hold on
plot(para0(1),para0(2),'x')
plot(para(1)+1.1774*x(1,:),para(2)+1.1774*x(2,:))
plot(para(1)+2.146*x(1,:),para(2)+2.146*x(2,:),'-.')
xlabel('K_1')
ylabel('K_3')
legend('Actual','Optimal Point','50%','90%')
grid