%****************************
% wtoK3d.m: 3D braced frame *
%****************************
clear
close all
clc
damagecase=0 % 0: undamaged, 1: damage case
Nd=18;       %  # DOFs
if damagecase==0
   modem=[1 2 4 5 7 9]; % modes to be observed for the undamaged structure
else
   modem=[1 2 4 5 6 7]; % modes to be observed for the damaged structure
end
Nm=length(modem);       % # modes considered
Nt=24;                  % # stiffness parameters
Nit=2000;               % # iterations
sen=[1:6 8 11 13:18];   % measured DOFs
Ns=length(sen);         % total number of measured DOFs
Lo0=zeros(Ns,Nd);
for p=1:Ns
    Lo0(p,sen(p))=1;
end
for m=1:Nm
    Lo((m-1)*Ns+1:m*Ns,(m-1)*Nd+1:m*Nd)=Lo0;
end
nw2=0.005;
nsh=0.005;
% Actual Mass and stiffness Matrix
m0=10/1000;a=5;
% Rotation matrix for formation of system matrices
for j=1:Nd/3
    T0((j-1)*3+1:j*3,(j-1)*3+1:j*3)=inv([1 0 -a/2;
                                         0 1 -a/2;
                                         1 0  a/2]);
end
Mmat=T0'*diag(repmat([m0 m0 m0/12*(2*a^2)],1,Nd/3))*T0; % Mass matrix
T0=T0(1:6,1:6);
% Formation of stiffness matrix
Kmat=zeros(Nd,Nd);
for j=1:Nd/3
    kpx=50;kpy=40;kmx=50;kmy=40;
    if (damagecase==1)*(j==1)
        kpx=42.5;
    end
    if (damagecase==1)*(j==4)
        kpy=36;
    end
    % Definition of substructure stiffness matrices
    Ktemp1=T0'*[0  0   0    0  0    0;
                0  1   a/2  0 -1   -a/2;
                0  a/2 0    0 -a/2  0;
                0  0   0    0  0    0;
                0 -1  -a/2  0  1    a/2;
                0 -a/2 0    0  a/2  0]*T0;
    Ktemp2=T0'*[1   0 -a/2 -1   0  a/2;
                0   0  0    0   0  0;
               -a/2 0  0    a/2 0  0;
               -1   0  a/2  1   0 -a/2;
                0   0  0    0   0  0;
                a/2 0  0   -a/2 0  0]*T0;
    Ktemp3=T0'*[0  0   0   0  0    0;
                0  1  -a/2 0 -1    a/2;
                0 -a/2 0   0 a/2   0;
                0  0   0   0  0    0;
                0 -1   a/2 0  1   -a/2;
                0 a/2  0   0  -a/2 0]*T0;
    Ktemp4=T0'*[1   0  a/2 -1   0 -a/2;
                0   0  0    0   0  0;
                a/2 0  0   -a/2 0  0;
               -1   0 -a/2  1   0  a/2;
                0   0  0    0   0  0;
               -a/2 0  0    a/2 0  0]*T0;
    Kt=a^2*((kpy-kmy)^2/4/(kpy+kmy)+(kpx-kmx)^2/4/(kpx+kmx)+kpy*kmy/(kpy+kmy)+kpx*kmx/(kpx+kmx));
    Ktemp0=T0'*[0  0  0  0  0  0;
                0  0  0  0  0  0;
                0  0  Kt 0  0 -Kt;
                0  0  0  0  0  0;
                0  0  0  0  0  0;
                0  0 -Kt 0  0  Kt]*T0;
    if j==1
       Kmat(1:3,1:3)=Kmat(1:3,1:3)+kpx*Ktemp1(1:3,1:3);
       Kmat(1:3,1:3)=Kmat(1:3,1:3)+kpy*Ktemp2(1:3,1:3);
       Kmat(1:3,1:3)=Kmat(1:3,1:3)+kmx*Ktemp3(1:3,1:3);
       Kmat(1:3,1:3)=Kmat(1:3,1:3)+kmy*Ktemp4(1:3,1:3);
       Kmat(1:3,1:3)=Kmat(1:3,1:3)+Ktemp0(1:3,1:3);
    else
       Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)=Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)+kpx*Ktemp1;
       Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)=Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)+kpy*Ktemp2;
       Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)=Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)+kmx*Ktemp3;
       Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)=Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)+kmy*Ktemp4;
       Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)=Kmat((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)+Ktemp0;
    end
end
[modesh0,w2r]=eig(inv(Mmat)*Kmat);
w2r=diag(w2r);[w2r,I]=sort(w2r);
wr=w2r.^0.5;
modesh0=modesh0(:,I);modesh=modesh0(:,modem);
for p=1:Nm
    modesh(:,p)=modesh(:,p)/norm(modesh(:,p)); % to normalize all modeshapes to have unit norm
end
disp(['Natural frequencies (Hz):' num2str(wr(1:6)'/2/pi)])

%***************************
% Generation of modal data *
%***************************
disp(['Data simulation in progress...'])
randn('seed',sum(999*clock));
w2m=w2r+nw2*w2r.*randn(Nd,1);
w2m=w2m(modem);

Sigbe=nsh^2/Nd*eye(Nm*(Ns+1));
for m=1:Nm
    Sigbe(m,m)=nw2^2*w2r(m)^2;
end
invSigbe=inv(Sigbe);
invSigbe11=invSigbe(1:Nm,1:Nm);invSigbe12=invSigbe(1:Nm,Nm+1:end);
invSigbe21=invSigbe(Nm+1:end,1:Nm);invSigbe22=invSigbe(Nm+1:end,Nm+1:end);
invSigth0=(100)^(-2)*eye(Nt);

randn('seed',sum(999*clock));
noise=nsh/Nd^0.5*randn(Ns-1,Nm);
phim=zeros(Ns,Nm);
for m=1:Nm
    [modeshtemp,I]=sort(modesh(sen,m));
    phim(:,m)=modesh(sen,m);
    phim(I(1:end-1),m)=phim(I(1:end-1),m)+noise(:,m);
    Sigbe(Nm+(m-1)*Ns+I(end),Nm+(m-1)*Ns+I(end))=1e-4*nsh^2/Nd;
end
phim=reshape(phim,Ns*Nm,1);
clear noise

%***********
% Model ID *
%***********
theta0=100*ones(Nt,1);
pa=[theta0 zeros(Nt,Nit)];
theta=theta0;w2=w2m;

Ki=zeros(Nd,Nd,Nt);
sigeq2=1; % Use a relatively large value for the eigen-equation error 
          % variance to accelerate the optimization at the beginning.
'Iteration in progress...'
tic
for it=1:Nit
    if it==100
       sigeq2=1e-4; % After 100 iterations, change the eigen-equation error
    end               % back to the tolerance level.
    K0=zeros(Nd);
    for j=1:Nd/3
        kpx=theta((j-1)*4+1);
        kpy=theta((j-1)*4+2);
        kmx=theta((j-1)*4+3);
        kmy=theta((j-1)*4+4);
        kt1=a^2*(2*(kpx^2-kmx^2)-(kpx-kmx)^2)/4/(kpx+kmx)^2+a^2*kmx^2/(kpx+kmx)^2;
        Kitemp1=T0'*[0  0   0    0  0    0;
                     0  1   a/2  0 -1   -a/2;
                     0  a/2 kt1  0 -a/2 -kt1;
                     0  0   0    0  0    0;
                     0 -1  -a/2  0  1    a/2;
                     0 -a/2 -kt1 0  a/2  kt1]*T0;
        kt2=a^2*(2*(kpy^2-kmy^2)-(kpy-kmy)^2)/4/(kpy+kmy)^2+a^2*kmy^2/(kpy+kmy)^2;
        Kitemp2=T0'*[1   0 -a/2 -1   0  a/2;
                     0   0  0    0   0  0;
                    -a/2 0  kt2  a/2 0 -kt2;
                    -1   0  a/2  1   0 -a/2;
                     0   0  0    0   0  0;
                     a/2 0 -kt2 -a/2 0  kt2]*T0;
        kt3=-a^2*(2*(kpx^2-kmx^2)+(kpx-kmx)^2)/4/(kpx+kmx)^2+a^2*kpx^2/(kpx+kmx)^2;
        Kitemp3=T0'*[0  0   0   0  0    0;
                     0  1  -a/2 0 -1    a/2;
                     0 -a/2 kt3 0 a/2  -kt3;
                     0  0   0   0  0    0;
                     0 -1   a/2 0  1   -a/2;
                     0 a/2 -kt3 0  -a/2 kt3]*T0;
        kt4=-a^2*(2*(kpy^2-kmy^2)+(kpy-kmy)^2)/4/(kpy+kmy)^2+a^2*kpy^2/(kpy+kmy)^2;
        Kitemp4=T0'*[1   0  a/2 -1   0 -a/2;
                     0   0  0    0   0  0;
                     a/2 0  kt4 -a/2 0  -kt4;
                    -1   0 -a/2  1   0  a/2;
                     0   0  0    0   0  0;
                    -a/2 0 -kt4 a/2 0  kt4]*T0;
        Kt=a^2*((kpy-kmy)^2/4/(kpy+kmy)+(kpx-kmx)^2/4/(kpx+kmx)+kpy*kmy/(kpy+kmy)+kpx*kmx/(kpx+kmx));
        Kc=kt1*kpx+kt2*kpy+kt3*kmx+kt4*kmy;
        K0temp=T0'*[0  0  0  0  0  0;
                0  0  0  0  0  0;
                0  0  Kt-Kc 0  0 -Kt+Kc;
                0  0  0  0  0  0;
                0  0  0  0  0  0;
                0  0 -Kt+Kc 0  0  Kt-Kc]*T0;
        if j==1
           Ki(1:3,1:3,1)=Kitemp1(1:3,1:3);
           Ki(1:3,1:3,2)=Kitemp2(1:3,1:3);
           Ki(1:3,1:3,3)=Kitemp3(1:3,1:3);
           Ki(1:3,1:3,4)=Kitemp4(1:3,1:3);
           K0(1:3,1:3)=K0temp(1:3,1:3);
        else
           Ki((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6,(j-1)*4+1)=Kitemp1;
           Ki((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6,(j-1)*4+2)=Kitemp2;
           Ki((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6,(j-1)*4+3)=Kitemp3;
           Ki((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6,(j-1)*4+4)=Kitemp4;
           K0((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)=K0((j-2)*3+1:(j-2)*3+6,(j-2)*3+1:(j-2)*3+6)+K0temp;
        end
    end
    Kk=K0;
    for j=1:Nt
        Kk=Kk+theta(j)*squeeze(Ki(:,:,j));
    end   
    % mode shapes expansion
    for m=1:Nm
        Gphi((m-1)*Nd+1:m*Nd,(m-1)*Nd+1:m*Nd)=(w2(m)*Mmat-Kk)^2;
    end
    phi=inv(1/sigeq2*Gphi+Lo'*invSigbe22*Lo)*Lo'*(invSigbe21*(w2m-w2)+invSigbe22*phim);
    % frequencies updating
    if it>10
       for m=1:Nm
           phip=phi((m-1)*Nd+1:m*Nd);
           Glambda(m,m)=phip'*Mmat^2*phip;
           Glambda2(m,1)=phip'*Mmat*Kk*phip;
       end
       w2=inv(1/sigeq2*Glambda+invSigbe11)*(1/sigeq2*Glambda2+invSigbe11*w2m+invSigbe12*(phim-Lo*phi));
    end
    % stiffness parameters updating
    for m=1:Nm
        phip=phi((m-1)*Nd+1:m*Nd);
        for j=1:Nt
            Gtheta((m-1)*Nd+1:m*Nd,j)=squeeze(Ki(:,:,j))*phip;
        end
        b((m-1)*Nd+1:m*Nd,1)=(w2(m)*Mmat-K0)*phip;
    end
    theta=inv(1/sigeq2*Gtheta'*Gtheta+invSigth0)*(1/sigeq2*Gtheta'*b+invSigth0*theta0);
    if min(theta)<0
       for j=1:Nt
           if theta(j)<0
              'warning!'
              theta(j)=pa(j,it);
           end
       end
    end
    pa(:,it+1)=theta;
end
toc
sigtp=zeros(Nd*Nm,Nt);
for m=1:Nm
    phip=phi((m-1)*Nd+1:m*Nd);
    for j=1:Nt
        sigtp((m-1)*Nd+1:m*Nd,j)=1/sigeq2*(squeeze(Ki(:,:,j))*(Kk-w2(m)*Mmat)+(Kk-w2(m)*Mmat)*squeeze(Ki(:,:,j)))*phip;
    end
end
sigtw=zeros(Nm,Nt);
for m=1:Nm
    phip=phi((m-1)*Nd+1:m*Nd);
    for j=1:Nt
         sigtw(m,j)=-1/sigeq2*phip'*Mmat*squeeze(Ki(:,:,j))*phip;
    end
end
sigwp=zeros(Nd*Nm,Nm);
for m=1:Nm
    sigwp((m-1)*Nd+1:m*Nd,m)=-2/sigeq2*(Kk-w2(m)*Mmat)*Mmat*phi((m-1)*Nd+1:m*Nd);
end
hess=[1/sigeq2*Glambda+invSigbe11  sigwp'+invSigbe12*Lo           sigtw;
      sigwp+Lo'*invSigbe21  1/sigeq2*Gphi+Lo'*invSigbe22*Lo       sigtp;
      sigtw'                       sigtp'                 1/sigeq2*Gtheta'*Gtheta+invSigth0];
SigALL=inv(hess);Sigth=SigALL(end-Nt+1:end,end-Nt+1:end);

results(:,1)=repmat([50;40],2*Nd/3,1);
if damagecase==1
   results([1 14],1)=[42.5;36];
end
results(:,2)=theta;
results(:,3)=diag(Sigth).^0.5;
results(:,4)=results(:,3)./results(:,2);
results(:,5)=abs(results(:,2)-results(:,1))./results(:,3);
disp(['  --------------------------------------------------'])
disp(['    Actual  Identified   s.d.     c.o.v.     n.d.'])
disp(['  --------------------------------------------------'])
disp([results])

if damagecase==0
   save wtoK3d_un
   'saved in wtoK3d_un'
end
if damagecase==1
   save wtoK3d_pd
   'saved in wtoK3d_pd'
end